@@ -26,6 +26,7 @@
 #include <string.h>
 
 #include <libavformat/avformat.h>
+#include <libavutil/imgutils.h>
 /* #include <ffmpeg/avi.h> */
 #include <gst/gst.h>
 #include <gst/base/gstflowcombiner.h>
@@ -922,18 +923,19 @@ gst_ffmpegdemux_get_stream (GstFFMpegDemux * demux, AVStream * avstream)
   gint num;
   gchar *padname;
   const gchar *codec;
-  AVCodecContext *ctx;
+  AVCodecContext *ctx = NULL;
   GstFFStream *stream;
   GstEvent *event;
   gchar *stream_id;
 
-  ctx = avstream->codec;
-
   oclass = (GstFFMpegDemuxClass *) G_OBJECT_GET_CLASS (demux);
 
   if (demux->streams[avstream->index] != NULL)
     goto exists;
 
+  ctx = avcodec_alloc_context3 (NULL);
+  avcodec_parameters_to_context (ctx, avstream->codecpar);
+
   /* create new stream */
   stream = g_new0 (GstFFStream, 1);
   demux->streams[avstream->index] = stream;
@@ -1042,23 +1044,27 @@ gst_ffmpegdemux_get_stream (GstFFMpegDemux * demux, AVStream * avstream)
         GST_TAG_VIDEO_CODEC : GST_TAG_AUDIO_CODEC, codec, NULL);
   }
 
+done:
+  if (ctx)
+    avcodec_free_context (&ctx);
   return stream;
 
   /* ERRORS */
 exists:
   {
     GST_DEBUG_OBJECT (demux, "Pad existed (stream %d)", avstream->index);
-    return demux->streams[avstream->index];
+    stream = demux->streams[avstream->index];
+    goto done;
   }
 unknown_type:
   {
     GST_WARNING_OBJECT (demux, "Unknown pad type %d", ctx->codec_type);
-    return stream;
+    goto done;
   }
 unknown_caps:
   {
     GST_WARNING_OBJECT (demux, "Unknown caps for codec %d", ctx->codec_id);
-    return stream;
+    goto done;
   }
 }
 
@@ -1483,12 +1489,12 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
   /* prepare to push packet to peer */
   srcpad = stream->pad;
 
-  rawvideo = (avstream->codec->codec_type == AVMEDIA_TYPE_VIDEO &&
-      avstream->codec->codec_id == AV_CODEC_ID_RAWVIDEO);
+  rawvideo = (avstream->codecpar->codec_type == AVMEDIA_TYPE_VIDEO &&
+      avstream->codecpar->codec_id == AV_CODEC_ID_RAWVIDEO);
 
   if (rawvideo)
-    outsize = gst_ffmpeg_avpicture_get_size (avstream->codec->pix_fmt,
-        avstream->codec->width, avstream->codec->height);
+    outsize = gst_ffmpeg_avpicture_get_size (avstream->codecpar->format,
+        avstream->codecpar->width, avstream->codecpar->height);
   else
     outsize = pkt.size;
 
@@ -1497,23 +1503,24 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
   /* copy the data from packet into the target buffer
    * and do conversions for raw video packets */
   if (rawvideo) {
-    AVPicture src, dst;
+    AVFrame src, dst;
     const gchar *plugin_name =
         ((GstFFMpegDemuxClass *) (G_OBJECT_GET_CLASS (demux)))->in_plugin->name;
     GstMapInfo map;
 
     GST_WARNING ("Unknown demuxer %s, no idea what to do", plugin_name);
     gst_ffmpeg_avpicture_fill (&src, pkt.data,
-        avstream->codec->pix_fmt, avstream->codec->width,
-        avstream->codec->height);
+        avstream->codecpar->format, avstream->codecpar->width,
+        avstream->codecpar->height);
 
     gst_buffer_map (outbuf, &map, GST_MAP_WRITE);
     gst_ffmpeg_avpicture_fill (&dst, map.data,
-        avstream->codec->pix_fmt, avstream->codec->width,
-        avstream->codec->height);
+        avstream->codecpar->format, avstream->codecpar->width,
+        avstream->codecpar->height);
 
-    av_picture_copy (&dst, &src, avstream->codec->pix_fmt,
-        avstream->codec->width, avstream->codec->height);
+    av_image_copy (dst.data, dst.linesize, (const uint8_t **) src.data,
+        src.linesize, avstream->codecpar->format, avstream->codecpar->width,
+        avstream->codecpar->height);
     gst_buffer_unmap (outbuf, &map);
   } else {
     gst_buffer_fill (outbuf, 0, pkt.data, outsize);
@@ -1960,7 +1967,7 @@ gboolean
 gst_ffmpegdemux_register (GstPlugin * plugin)
 {
   GType type;
-  AVInputFormat *in_plugin;
+  const AVInputFormat *in_plugin;
   gchar *extensions;
   GTypeInfo typeinfo = {
     sizeof (GstFFMpegDemuxClass),
@@ -1974,11 +1981,11 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
     (GInstanceInitFunc) gst_ffmpegdemux_init,
   };
 
-  in_plugin = av_iformat_next (NULL);
+  void *i = 0;
 
   GST_LOG ("Registering demuxers");
 
-  while (in_plugin) {
+  while ((in_plugin = av_demuxer_iterate (&i))) {
     gchar *type_name, *typefind_name;
     gint rank;
     gboolean register_typefind_func = TRUE;
@@ -2006,14 +2013,14 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
         !strncmp (in_plugin->name, "f64", 3) ||
         !strcmp (in_plugin->name, "mulaw") || !strcmp (in_plugin->name, "alaw")
         )
-      goto next;
+      continue;
 
     /* no network demuxers */
     if (!strcmp (in_plugin->name, "sdp") ||
         !strcmp (in_plugin->name, "rtsp") ||
         !strcmp (in_plugin->name, "applehttp")
         )
-      goto next;
+      continue;
 
     /* these don't do what one would expect or
      * are only partially functional/useful */
@@ -2021,7 +2028,7 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
         !strcmp (in_plugin->name, "wv") ||
         !strcmp (in_plugin->name, "ass") ||
         !strcmp (in_plugin->name, "ffmetadata"))
-      goto next;
+      continue;
 
     /* Don't use the typefind functions of formats for which we already have
      * better typefind functions */
@@ -2096,12 +2103,14 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
         !strcmp (in_plugin->name, "mpc8") ||
         !strcmp (in_plugin->name, "ivf") ||
         !strcmp (in_plugin->name, "brstm") ||
-        !strcmp (in_plugin->name, "bfstm") || !strcmp (in_plugin->name, "gif"))
+        !strcmp (in_plugin->name, "bfstm") ||
+        !strcmp (in_plugin->name, "gif") ||
+        !strcmp (in_plugin->name, "dsf") || !strcmp (in_plugin->name, "iff"))
       rank = GST_RANK_MARGINAL;
     else {
       GST_DEBUG ("ignoring %s", in_plugin->name);
       rank = GST_RANK_NONE;
-      goto next;
+      continue;
     }
 
     /* construct the type */
@@ -2111,7 +2120,7 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
     /* if it's already registered, drop it */
     if (g_type_from_name (type_name)) {
       g_free (type_name);
-      goto next;
+      continue;
     }
 
     typefind_name = g_strdup_printf ("avtype_%s", in_plugin->name);
@@ -2129,8 +2138,8 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
     if (!gst_element_register (plugin, type_name, rank, type) ||
         (register_typefind_func == TRUE &&
             !gst_type_find_register (plugin, typefind_name, rank,
-                gst_ffmpegdemux_type_find, extensions, NULL, in_plugin,
-                NULL))) {
+                gst_ffmpegdemux_type_find, extensions, NULL,
+                (gpointer) in_plugin, NULL))) {
       g_warning ("Registration of type %s failed", type_name);
       g_free (type_name);
       g_free (typefind_name);
@@ -2141,9 +2150,6 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
     g_free (type_name);
     g_free (typefind_name);
     g_free (extensions);
-
-  next:
-    in_plugin = av_iformat_next (in_plugin);
   }
 
   GST_LOG ("Finished registering demuxers");
